#   File:       SampleModule.make
#   Target:     SampleModule
#   Sources:    SampleServerModule.c
#				SampleClientModule.c
#				SampleUserConfigModule.c
#				SampleSecurity.r
#
#	This is the make file for the Sample Add-on Security module for Apple Remote
#	Access (ARA).  It builds a module that allows a second password to be assigned to each user.
#	To build it use the MPW Build command.  The resulting module will be named "SampleModule".
#	You must place this module in the extensions folders of both the client and the server.
#	The SecurityInterfaces and SecurityLibraries folders (on Captain Crunch developer disk) must
#	be placed in the folder that contains this folder in order for this make file to work
#	properly.
#
#	Copyright  1993, Apple Computer, Inc.
#	Author:  Farzad Sarabi
#
#   Created:    Wednesday, February 3, 1993 11:03:43 AM

SECURITYLIBRARIES = ::SecurityLibraries:	# where security related libraries are
SECURITYINCLUDES = ::SecurityInterfaces:	# where security related headers are
OBJ_DIR = :objects:							# where our object files get placed

# NOTE:
#	SecurityInterface.o must be the first file to be linked.

# objects for building the server-side authentication code resource
SERVER_OBJS =	{SECURITYLIBRARIES}SecurityInterface.o 
				{OBJ_DIR}SampleServerModule.c.o 
				{Libraries}Runtime.o

# objects for building client-side authentication code resource
CLIENT_OBJS =	{SECURITYLIBRARIES}SecurityInterface.o 
				{OBJ_DIR}SampleClientModule.c.o 
				{Libraries}Runtime.o

# objects for building server-side user configuration code resource
CONFIG_OBJS =	{SECURITYLIBRARIES}SecurityInterface.o 
				{OBJ_DIR}SampleUserConfigModule.c.o 
				{Libraries}Runtime.o

# builds the modules resources
SampleModule  SampleModule.make SampleSecurity.r
	Rez SampleSecurity.r -i {SECURITYINCLUDES} -append -o SampleModule

# first one for building the server-side authentication
SampleModule  SampleModule.make {SERVER_OBJS}
	Link -t CLMP -c SMPL -rt SRPR=200 -m SECURITYINTERFACEENTRYPROC -sg SampleModule 
		{SERVER_OBJS} 
		-o SampleModule

# now build the client-side authentication code resource
SampleModule  SampleModule.make {CLIENT_OBJS}
	Link -t CLMP -c SMPL -rt CLPR=200 -m SECURITYINTERFACEENTRYPROC -sg SampleModule 
		{CLIENT_OBJS} 
		-o SampleModule

# now build the server-side user configuration module
SampleModule  SampleModule.make {CONFIG_OBJS}
	Link -t CLMP -c SMPL -rt UCNF=200 -m SECURITYINTERFACEENTRYPROC -sg SampleModule 
		{CONFIG_OBJS} 
		-o SampleModule

# compile the c files
{OBJ_DIR}SampleServerModule.c.o  SampleModule.make SampleServerModule.c
	 C -r -b SampleServerModule.c -i {SECURITYINCLUDES} -o {OBJ_DIR}

{OBJ_DIR}SampleClientModule.c.o  SampleModule.make SampleClientModule.c
	 C -r -b SampleClientModule.c -i {SECURITYINCLUDES} -o {OBJ_DIR}

{OBJ_DIR}SampleUserConfigModule.c.o  SampleModule.make SampleUserConfigModule.c
	 C -r -b SampleUserConfigModule.c -i {SECURITYINCLUDES} -o {OBJ_DIR}

